@@ -10,6 +10,9 @@ from openflash.results import Results
 from scipy import linalg
 from openflash.multi_constants import *
 from functools import partial
+from openflash.body import SteppedBody
+from openflash.geometry import ConcentricBodyGroup
+from openflash.basic_region_geometry import BasicRegionGeometry
 
 class MEEMEngine:
     """
@@ -50,6 +53,7 @@ class MEEMEngine:
         """
         Assemble the system matrix A for a given problem using pre-computed blocks.
         """
+        self._ensure_m_k_and_N_k_arrays(problem, m0)
         cache = self.cache_list[problem]
         A = cache._get_A_template()
 
@@ -65,6 +69,7 @@ class MEEMEngine:
         """
         Assemble the right-hand side vector b for a given problem.
         """
+        self._ensure_m_k_and_N_k_arrays(problem, m0)
         cache = self.cache_list[problem]
         b = cache._get_b_template()
 
@@ -115,7 +120,6 @@ class MEEMEngine:
             vals = np.zeros((NMK[boundary_count - 1], NMK[boundary_count]), dtype=complex)
             for m in range(NMK[boundary_count - 1]):
                 for k in range(NMK[boundary_count]):
-                    # FIX: Consistent argument order (h, d)
                     vals[m, k] = I_mk(m, k, boundary_count - 1, d, m0, h, m_k_arr, N_k_arr)
             return vals
         
@@ -137,7 +141,6 @@ class MEEMEngine:
                 
                 if bd > 0:
                     left_block2 = p_diagonal_block(True, R_2n_func, bd, h, d, a, NMK)
-                    # FIX: Correct order is R1n block, then R2n block
                     block = np.concatenate([left_block1, left_block2], axis=1)
                 else:
                     block = left_block1
@@ -158,7 +161,6 @@ class MEEMEngine:
                 row_height = N if left_diag else M
                 blocks = []
                 
-                # FIX: Correctly ordered R1n/R2n blocks
                 if left_diag:
                     blocks.append(p_diagonal_block(True, R_1n_func, bd, h, d, a, NMK))
                     if bd > 0: blocks.append(p_diagonal_block(True, R_2n_func, bd, h, d, a, NMK))
@@ -194,7 +196,6 @@ class MEEMEngine:
                         cache._add_m0_dependent_A_entry(g_row, g_col, calc_func)
                     # Second dense block (R2n) if needed
                     if bd > 0:
-                        # FIX: Correctly place the R2n block after the R1n block
                         r2n_col_start = v_dense_e_col_start + N
                         for k_local in range(N):
                             g_row, g_col = row_offset + m_local, r2n_col_start + k_local
@@ -205,7 +206,6 @@ class MEEMEngine:
                 v_diag_e_col_start = col_offset + (2*N if bd > 0 else N)
                 for k_local in range(M):
                     g_row, g_col = row_offset + k_local, v_diag_e_col_start + k_local
-                    # FIX: Removed unused 'm' argument from lambda signature
                     calc_func = lambda p, m0, mk, Nk, Imk, k=k_local: \
                         v_diagonal_block_e_entry(m, k, bd, m0, mk, a, h)
                     cache._add_m0_dependent_A_entry(g_row, g_col, calc_func)
@@ -216,7 +216,6 @@ class MEEMEngine:
                 row_height = N if left_diag else M
                 blocks = []
                 
-                # FIX: Correctly ordered diff_R1n/diff_R2n blocks
                 if left_diag:
                     blocks.append(v_diagonal_block(True, diff_R_1n_func, bd, h, d, NMK, a))
                     if bd > 0: blocks.append(v_diagonal_block(True, diff_R_2n_func, bd, h, d, NMK, a))
@@ -310,13 +309,16 @@ class MEEMEngine:
         cs.append(x[row:])
         return cs
 
-    def compute_hydrodynamic_coefficients(self, problem, X, m0):
+    def compute_hydrodynamic_coefficients(self, problem, X, m0, modes_to_calculate: np.ndarray = None):
         """
         Compute hydrodynamic coefficients (added mass and damping) for each mode defined in the problem.
 
         :param problem: Problem object containing the geometry
         :param X: Solution vector from eigenfunction solver
         :param m0: Mode number (used for omega and excitation terms)
+        :param modes_to_calculate: A list of all body indices to calculate forces *for*.
+                                   If None, defaults to `problem.modes` (which infers from the
+                                   problem's *own* geometry's heaving flags).
         :return: List of dictionaries, one per mode, with hydrodynamic properties
         """
         geometry = problem.geometry
@@ -334,9 +336,24 @@ class MEEMEngine:
 
         results_per_mode = []
 
+        # Determine which modes to loop over to calculate forces on
+        if modes_to_calculate is None:
+            # OLD LINE:
+            # modes_to_calculate = problem.modes
+            
+            # NEW, CORRECTED LOGIC:
+            # Default to *all* body indices. The 'problem.modes' property
+            # reflects the 'heaving' flags used to generate the potential 'X' (the "source" mode i).
+            # For the force calculation, we need to iterate over *all* modes 'j'
+            # (including stationary ones) to get the full column (A_ji, B_ji) of the matrix.
+            num_bodies = len(geometry.body_arrangement.bodies)
+            modes_to_calculate = np.arange(num_bodies)
+
         # Loop through each mode (degree of freedom)
-        for mode_index in range(len(domain_keys) - 1):
+        for mode_index in modes_to_calculate:
             # Set heaving vector: only one mode active at a time
+            # This 'heaving' vector is a hypothetical one for the force calculation
+            # (using Haskind relations), NOT the heaving vector that generated the potential X.
             heaving = [0] * len(domain_keys)
             heaving[mode_index] = 1
 
@@ -408,19 +425,27 @@ class MEEMEngine:
         m_k_arr = cache.m_k_arr
         N_k_arr = cache.N_k_arr
         
+        # Get geometry parameters directly from the body arrangement and domains
+        geometry = problem.geometry
+        body_arrangement = geometry.body_arrangement
         domain_list = problem.domain_list
+        
+        # These are the correct physical parameters for meshgrid and particular solution
+        a = body_arrangement.a
+        d = body_arrangement.d
+        heaving = body_arrangement.heaving
+        
+        # These are needed for the homogeneous solution and coefficient reformatting
+        h = geometry.h
         domain_keys = list(domain_list.keys())
         boundary_count = len(domain_keys) - 1
-
         NMK = [domain_list[idx].number_harmonics for idx in domain_keys]
-        h = domain_list[0].h
-        d = [domain_list[idx].di for idx in domain_keys if domain_list[idx].di is not None]
-        a = [domain_list[idx].a for idx in domain_keys if domain_list[idx].a is not None]
-        heaving = [domain_list[idx].heaving for idx in domain_keys]
         
+        # --- The rest of the function remains the same ---
         Cs = self.reformat_coeffs(solution_vector, NMK, boundary_count)
 
-        # --- 2. Create Meshgrid and Regions ---
+        # 2. Create Meshgrid and Regions
+        # Now make_R_Z will receive the correct a and d lists
         R, Z = make_R_Z(a, h, d, sharp, spatial_res)
         regions = []
         regions.append((R <= a[0]) & (Z < -d[0]))
@@ -502,26 +527,31 @@ class MEEMEngine:
     def calculate_velocities(self, problem, solution_vector: np.ndarray, m0, spatial_res, sharp) -> Dict[str, Any]:
         """
         Calculate full spatial velocities vr and vz on a meshgrid for visualization.
-        (Optimized to pre-compute components and use vectorized operations)
         """
         self._ensure_m_k_and_N_k_arrays(problem, m0)
         cache = self.cache_list[problem]
         m_k_arr, N_k_arr = cache.m_k_arr, cache.N_k_arr
 
-        # --- 1. Extract parameters and pre-calculate coefficients ONCE ---
+        geometry = problem.geometry
+        body_arrangement = geometry.body_arrangement
         domain_list = problem.domain_list
+
+        # Get physical body parameters for meshgrid and particular solution
+        a = body_arrangement.a
+        d = body_arrangement.d
+        heaving = body_arrangement.heaving
+        
+        # Get domain/solver parameters for homogeneous solution and coefficient reformatting
+        h = geometry.h
         domain_keys = list(domain_list.keys())
         boundary_count = len(domain_keys) - 1
-
         NMK = [domain_list[idx].number_harmonics for idx in domain_keys]
-        h = domain_list[0].h
-        d = [domain_list[idx].di for idx in domain_keys if domain_list[idx].di is not None]
-        a = [domain_list[idx].a for idx in domain_keys if domain_list[idx].a is not None]
-        heaving = [domain_list[idx].heaving for idx in domain_keys]
 
+        # --- The rest of the function remains the same ---
         Cs = self.reformat_coeffs(solution_vector, NMK, boundary_count)
 
-        # --- 2. Create Meshgrid and Regions ---
+        # 2. Create Meshgrid and Regions
+        # This will now use the correct 'a' and 'd' arrays
         R, Z = make_R_Z(a, h, d, sharp, spatial_res)
         regions = [
             (R <= a[0]) & (Z < -d[0]),
@@ -584,77 +614,129 @@ class MEEMEngine:
         """
         Perform the full MEEM computation for all frequencies defined in the selected MEEMProblem,
         and store results in a `Results` object.
+        
+        This method correctly solves the N-body radiation problem by:
 
-        The method:
-        - Assembles and solves the linear system A @ X = b for each frequency in `problem.frequencies`.
-        - Computes hydrodynamic coefficients (added mass and damping).
-        - Stores domain potentials.
-        - Stores all results in a reusable `Results` object.
-
-        Parameters
-        ----------
-        problem_index : int
-            Index of the MEEMProblem instance from `self.problem_list` to run computations for.
-
-        Returns
-        -------
-        Results
-            A `Results` object containing added mass and damping matrices, 
-            and optionally, computed potentials for each frequency and mode.
+        - Looping through each frequency.
+        - Looping through each radiating mode `i` (inferred from the problem's heaving flags).
+        - Creating a temporary problem where only body `i` heaves.
+        - Solving for the potential `X_i` from this single radiation.
+        - Calculating the forces on all bodies `j` from `X_i` to get column `i`
+          of the hydrodynamic matrices (`A_ji`, `B_ji`).
+        - Storing the full N x N matrices.
+        - Storing the potential coefficients `Cs_i` for each radiation problem.
         """
-        problem = self.problem_list[problem_index]
-        geometry = problem.geometry
-        results = Results(geometry, problem.frequencies, problem.modes)
+        # 1. Get original problem setup
+        original_problem = self.problem_list[problem_index]
+        original_geometry = original_problem.geometry
+        original_bodies = original_geometry.body_arrangement.bodies
+        h = original_geometry.h
+        
+        # Get NMK list from the original problem's domains
+        original_domain_list = original_problem.domain_list
+        original_domain_keys = list(original_domain_list.keys())
+        NMK_list = [original_domain_list[idx].number_harmonics for idx in original_domain_keys]
 
-        num_modes = len(problem.modes)
-        num_freqs = len(problem.frequencies)
+        # Get modes and frequencies from the problem
+        # problem.modes is now a property that infers from geometry's heaving flags
+        problem_modes = original_problem.modes
+        omegas_to_run = original_problem.frequencies
+        
+        num_modes = len(problem_modes)
+        num_freqs = len(omegas_to_run)
 
-        full_added_mass_matrix = np.full((num_freqs, num_modes), np.nan)
-        full_damping_matrix = np.full((num_freqs, num_modes), np.nan)
+        # Initialize Results object (constructor no longer takes modes)
+        results = Results(original_problem)
+
+        # Initialize 3D arrays to hold the (N x N) matrices for each frequency
+        full_added_mass_matrix = np.full((num_freqs, num_modes, num_modes), np.nan)
+        full_damping_matrix = np.full((num_freqs, num_modes, num_modes), np.nan)
         all_potentials_batch_data = []
 
-        for freq_idx, m0 in enumerate(problem.frequencies):
-            self._ensure_m_k_and_N_k_arrays(problem, m0)
+        # --- Loop 1: Over all frequencies ---
+        for freq_idx, omega in enumerate(omegas_to_run):
+            m0 = wavenumber(omega, h)
+            
+            # --- Loop 2: Over all radiating modes (i) ---
+            # We must solve one radiation problem for each moving body
+            for i_idx, radiating_mode in enumerate(problem_modes):
+                
+                try:
+                    # 1. Create a new geometry where ONLY this body is heaving
+                    temp_bodies = []
+                    for body_j, original_body in enumerate(original_bodies):
+                        # This check assumes body index corresponds to mode index
+                        is_heaving = (body_j == radiating_mode)
+                        temp_bodies.append(
+                            SteppedBody(
+                                a=original_body.a, 
+                                d=original_body.d, 
+                                slant_angle=original_body.slant_angle, 
+                                heaving=is_heaving
+                            )
+                        )
+                    
+                    # 2. Create the new problem for this single radiating body
+                    temp_arrangement = ConcentricBodyGroup(temp_bodies)
+                    temp_geometry = BasicRegionGeometry(temp_arrangement, h=h, NMK=NMK_list)
+                    temp_problem = MEEMProblem(temp_geometry)
+                    
+                    # Frequencies: Just this one.
+                    # set_frequencies_modes is now set_frequencies
+                    temp_problem.set_frequencies(np.array([omega]))
+                    
+                    # 3. Create a temporary engine to build the cache
+                    # This builds the 'b' vector correctly for only body 'i' radiating
+                    temp_engine = MEEMEngine(problem_list=[temp_problem])
+                
+                    # 4. Solve the system for this single mode's potential (X_i)
+                    X_i = temp_engine.solve_linear_system_multi(temp_problem, m0)
+                    
+                    # 5. Calculate forces on ALL modes (j) due to this potential (X_i)
+                    # This returns one column of the matrix
+                    # We must pass 'modes_to_calculate' so it knows all modes from the *original* problem
+                    hydro_coeffs_col = temp_engine.compute_hydrodynamic_coefficients(
+                        temp_problem, X_i, m0, modes_to_calculate=problem_modes
+                    )
+                    
+                    # 6. Populate the full matrices
+                    for coeff_dict in hydro_coeffs_col:
+                        j_mode = coeff_dict['mode'] # This is the 'j' index (force)
+                        j_idx = np.where(problem_modes == j_mode)[0][0]
+                        
+                        # A_ji (force on j, motion from i)
+                        full_added_mass_matrix[freq_idx, j_idx, i_idx] = coeff_dict['real']
+                        # B_ji (force on j, motion from i)
+                        full_damping_matrix[freq_idx, j_idx, i_idx] = coeff_dict['imag']
 
-            try:
-                A = self.assemble_A_multi(problem, m0)
-                b = self.assemble_b_multi(problem, m0)
-                X = np.linalg.solve(A, b)
-            except np.linalg.LinAlgError as e:
-                print(f"  ERROR: Could not solve for m0={m0:.4f}: {e}. Storing NaN for coefficients.")
-                continue
+                    # 7. Store the computed potential coefficients (Cs)
+                    Cs = temp_engine.reformat_coeffs(X_i, NMK_list, len(NMK_list) - 1)
+                    current_mode_potentials = {}
+                    for domain_idx, domain in enumerate(temp_problem.geometry.fluid_domains):
+                        domain_coeffs = Cs[domain_idx]
+                        current_mode_potentials[domain.index] = {
+                            "potentials": domain_coeffs,
+                            "r_coords_dict": {f"r_h{k}": 0.0 for k in range(len(domain_coeffs))}, # Placeholder
+                            "z_coords_dict": {f"z_h{k}": 0.0 for k in range(len(domain_coeffs))}  # Placeholder
+                        }
 
-            hydro_coeffs = self.compute_hydrodynamic_coefficients(problem, X, m0)
-            added_mass = np.array([hc["real"] for hc in hydro_coeffs])
-            damping = np.array([hc["imag"] for hc in hydro_coeffs])
-
-            if added_mass.shape[0] != num_modes or damping.shape[0] != num_modes:
-                raise ValueError(
-                    f"compute_hydrodynamic_coefficients returned shape mismatch for m0={m0:.4f}."
-                )
-
-            full_added_mass_matrix[freq_idx, :] = added_mass
-            full_damping_matrix[freq_idx, :] = damping
-
-            for mode_idx, _ in enumerate(problem.modes):
-                current_mode_potentials = {}
-                for domain in problem.geometry.domain_list.values():
-                    nh = domain.number_harmonics
-                    current_mode_potentials[domain.category] = {
-                        "potentials": (np.random.rand(nh) + 1j * np.random.rand(nh)).astype(complex),
-                        "r_coords_dict": {f"r_h{k}": np.random.rand() for k in range(nh)},
-                        "z_coords_dict": {f"z_h{k}": np.random.rand() for k in range(nh)},
-                    }
-
-                all_potentials_batch_data.append({
-                    "frequency_idx": freq_idx,
-                    "mode_idx": mode_idx,
-                    "data": current_mode_potentials,
-                })
+                    all_potentials_batch_data.append({
+                        "frequency_idx": freq_idx,
+                        "mode_idx": i_idx, # Index of the radiating mode
+                        "data": current_mode_potentials,
+                    })
+                        
+                except np.linalg.LinAlgError as e:
+                    print(f"  ERROR: Could not solve for freq={omega:.4f}, mode={radiating_mode}: {e}. Storing NaN.")
+                    # Mark this column as NaN
+                    full_added_mass_matrix[freq_idx, :, i_idx] = np.nan
+                    full_damping_matrix[freq_idx, :, i_idx] = np.nan
+                    continue # Go to the next radiating mode
 
+        # --- Store all results after loops are complete ---
+        # store_hydrodynamic_coefficients no longer takes modes
         results.store_hydrodynamic_coefficients(
-            frequencies=problem.frequencies,
-            modes=problem.modes,
+            frequencies=omegas_to_run,
             added_mass_matrix=full_added_mass_matrix,
             damping_matrix=full_damping_matrix,
         )
@@ -662,4 +744,4 @@ class MEEMEngine:
         if all_potentials_batch_data:
             results.store_all_potentials(all_potentials_batch_data)
 
-        return results
+        return results
\ No newline at end of file
