@@ -18,23 +18,33 @@ class MEEMProblem:
         self.geometry = geometry
         self.domain_list = geometry.domain_list
         self.frequencies = np.array([])  # Angular frequencies ω (rad/s)
-        self.modes = np.array([])        # Mode numbers (e.g. [0, 1, 2])
 
-    def set_frequencies_modes(self, frequencies: np.ndarray, modes: np.ndarray):
+    def set_frequencies(self, frequencies: np.ndarray):
         """
-        Set the angular frequencies and degrees of freedom for this problem.
+        Set the angular frequencies for this problem.
 
         Parameters
         ----------
         frequencies : np.ndarray
             Array of angular frequencies or omega (rad/s).
-        modes : np.ndarray
-            Array of degrees of freedom indices.
         """
-        assert isinstance(modes, np.ndarray), "modes must be a numpy array"
         assert np.all(frequencies > 0), "All frequencies must be positive"
         assert isinstance(frequencies, np.ndarray), "frequencies must be a numpy array"
-        assert np.all(modes >= 0), "modes must be non-negative integers"
 
         self.frequencies = frequencies
-        self.modes = modes
+
+    @property
+    def modes(self) -> np.ndarray:
+        """
+        Infers the active modes (degrees of freedom) from the
+        heaving flags set on the geometry's bodies.
+        
+        This assumes the body index (0, 1, 2...) corresponds
+        to the mode index.
+        """
+        # Assumes bodies are SteppedBody or CoordinateBody
+        heaving_bodies = [
+            i for i, body in enumerate(self.geometry.body_arrangement.bodies)
+            if body.heaving
+        ]
+        return np.array(heaving_bodies)
\ No newline at end of file
